﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Net;


namespace pmOne.PowerBI.PowerBIObjects
{
    [DataContract]
    public class PBIDashboard : PBIObject
    {
        #region Constructors
        #endregion
        #region Private Properties for Serialization
        [DataMember(Name = "@odata.context", IsRequired = false, EmitDefaultValue = false)]
        private string ODataContext;

        [DataMember(Name = "id", IsRequired = true)]
        private Guid _id;

        [DataMember(Name = "displayName", IsRequired = true)]
        private string _displayName;

        [DataMember(Name = "isReadOnly", IsRequired = false)]
        private bool _isReadOnly;

        private List<PBITile> _tiles;
        #endregion

        #region Public Properties
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        public string DisplayName
        {
            get { return _displayName; }
            set { _displayName = value; }
        }

        public bool IsReadOnly
        {
            get { return _isReadOnly; }
            set { _isReadOnly = value; }
        }
        
        public List<PBITile> Tiles
        {
            get
            {
                if (_tiles == null && ID != null)
                {
                    LoadTilesFromPowerBI();
                }
                return _tiles;
            }
            set
            {
                foreach (PBITile tbl in value)
                {
                    tbl.ParentDashboard = this;
                }
                _tiles = value;
            }
        }

        public PBITile GetTileByName(string name)
        {
            foreach (PBITile tile in Tiles)
            {
                if (tile.Title == name)
                    return tile;
            }

            return null;
        }

        public PowerBIClient ParentPowerBIAPI { get; set; }

        #endregion
        
        public void LoadTilesFromPowerBI()
        {
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(PBITiles));

            PBITiles pbitls = (PBITiles)serializer.ReadObject(ParentPowerBIAPI.SendGETRequest("dashboards/" + ID + "/tiles").GetResponseStream());

            Tiles = pbitls.Tiles;
        }
    }
}
