﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Net;


namespace pmOne.PowerBI.PowerBIObjects
{
    [DataContract]
    public class PBIDataset : PBIObject
    {
        #region Constructors
        public PBIDataset(string name)
        {
            Name = name;

            _tables = new List<PBITable>();
        }
        #endregion
        #region Private Properties for Serialization
        [DataMember(Name = "@odata.context", IsRequired = false, EmitDefaultValue = false)]
        private string ODataContext;

        [DataMember(Name = "id", EmitDefaultValue = false)]
        private Guid _id;

        [DataMember(Name = "name", IsRequired = true)]
        private string _name;

        [DataMember(Name = "tables", IsRequired = false)]
        private List<PBITable> _tables;

        [DataMember(Name = "defaultRetentionPolicy", IsRequired = false, EmitDefaultValue = false)]
        private string _defaultRetentionPolicyString;
        #endregion

        #region Public Properties
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public List<PBITable> Tables
        {
            get
            {
                if (_tables == null && ID != null)
                {
                    LoadTablesFromPowerBI();
                }
                return _tables;
            }
            set
            {
                foreach (PBITable tbl in value)
                {
                    tbl.ParentDataset = this;
                }
                _tables = value;
            }
        }

        public PBITable GetTableByName(string name)
        {
            PBITable ret = null;

            foreach (PBITable tbl in Tables)
            {
                if (tbl.Name == name)
                    ret = tbl;
            }

            return ret;
        }

        public PowerBIClient_DefaultRetentionPolicy DefaultRetentionPolicy
        {
            get
            {
                return (PowerBIClient_DefaultRetentionPolicy)Enum.Parse(typeof(PowerBIClient_DefaultRetentionPolicy), _defaultRetentionPolicyString);
            }
            set
            {
                _defaultRetentionPolicyString = value.ToString();
            }
        }

        public PowerBIClient ParentPowerBIAPI { get; set; }

        #endregion
        public void LoadTablesFromPowerBI()
        {
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(PBITables));

            PBITables pbitbls = (PBITables)serializer.ReadObject(ParentPowerBIAPI.SendGETRequest("datasets/" + ID + "/tables").GetResponseStream());

            Tables = pbitbls.Tables;
        }

        public Guid PublishToPowerBI(PowerBIClient powerBiAPI)
        {
            return PublishToPowerBI(powerBiAPI, PowerBIClient_DefaultRetentionPolicy.None);
        }
        public Guid PublishToPowerBI(PowerBIClient powerBiAPI, PowerBIClient_DefaultRetentionPolicy defaultRetentionPolicy)
        {
            using (HttpWebResponse response = powerBiAPI.SendPOSTRequest("datasets?defaultRetentionPolicy=" + defaultRetentionPolicy.ToString(), JSON))
            {
                using (StreamReader streamReader = new StreamReader(response.GetResponseStream(), true))
                {
                    string json = streamReader.ReadToEnd();

                    ID = Guid.Parse(Regex.Match(json, "\"id\"\\s*:\\s*\"([a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12})\"").Groups[1].Value);
                    ODataContext = Regex.Match(json, "\"@odata.context\"\\s*:\\s*\"([^\"]*)\"").Groups[1].Value;
                    DefaultRetentionPolicy = (PowerBIClient_DefaultRetentionPolicy)Enum.Parse(typeof(PowerBIClient_DefaultRetentionPolicy), Regex.Match(json, "\"defaultRetentionPolicy\"\\s*:\\s*\"([^\"]*)\"").Groups[1].Value);
                }
            }

            return ID;
        }
    }
}
