﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Threading.Tasks;

namespace pmOne.PowerBI.PowerBIObjects
{
    [DataContract]
    public class PBITable : PBIObject
    {
        #region Constructors
        public PBITable(string name)
        {
            Name = name;

            Columns = new List<PBIColumn>();
        }
        #endregion
        #region Private Properties for Serialization
        [DataMember(Name = "name", IsRequired = true)]
        private string _name;

        [DataMember(Name = "columns", EmitDefaultValue = false)]
        private List<PBIColumn> _columns;
        #endregion
        #region Public Properties
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public List<PBIColumn> Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        public PBIDataset ParentDataset { get; set; }
        #endregion

        public void PublishToPowerBI(PowerBIClient powerBiAPI)
        {
            if (ParentDataset == null)
                throw new Exception("Cannot Publish Table to PowerBI as it is not linked to a DataSet in PowerBI!");

            using (HttpWebResponse response = powerBiAPI.SendPUTRequest(string.Format("datasets/{0}/tables/{1}", ParentDataset.ID.ToString(), Name), JSON))
            {
                using (StreamReader streamReader = new StreamReader(response.GetResponseStream(), true))
                {
                    string json = streamReader.ReadToEnd();
                }
            }
        }

        public PBIRow GetSampleRow()
        {
            PBIRow ret = new PBIRow();
            if (Columns != null)
            {
                foreach (PBIColumn col in Columns)
                {
                    ret.Add(col.Name, null);
                }
            }

            ret.ParentTable = this;

            return ret;
        }

        private void AddRowsToPowerBI(PowerBIClient powerBiAPI, string JSON)
        {
            if (ParentDataset == null)
                throw new Exception("Cannot add row to PowerBI table as the table is not linked to a DataSet in PowerBI!");

            if (Name == null)
                throw new Exception("Cannot add row to PowerBI table as the table does not have a name!");

            using (HttpWebResponse response = powerBiAPI.SendPOSTRequest(string.Format("datasets/{0}/tables/{1}/rows", ParentDataset.ID.ToString(), Name), JSON))
            {
                using (StreamReader streamReader = new StreamReader(response.GetResponseStream(), true))
                {
                    string json = streamReader.ReadToEnd();
                }
            }
        }

        public void AddRowToPowerBI(PowerBIClient powerBiAPI, PBIRow row)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("{");
            sb.AppendLine("\"rows\": [");

            sb.AppendLine(row.JSON);

            sb.AppendLine("]}");

            AddRowsToPowerBI(powerBiAPI, sb.ToString());
        }

        public void AddRowsToPowerBI(PowerBIClient powerBiAPI, PBIRows rows)
        {
            AddRowsToPowerBI(powerBiAPI, rows.JSON);
        }

        public void DeleteRowsFromPowerBI(PowerBIClient powerBiAPI)
        {
            if (ParentDataset == null)
                throw new Exception("Cannot add row to PowerBI table as the table is not linked to a DataSet in PowerBI!");

            if (Name == null)
                throw new Exception("Cannot add row to PowerBI table as the table does not have a name!");

            using (HttpWebResponse response = powerBiAPI.SendDELETERequest(string.Format("datasets/{0}/tables/{1}/rows", ParentDataset.ID.ToString(), Name)))
            {
                using (StreamReader streamReader = new StreamReader(response.GetResponseStream(), true))
                {
                    string json = streamReader.ReadToEnd();
                }
            }
        }
    }
}
